import pygame
from pygame.constants import HWSURFACE, DOUBLEBUF, RESIZABLE
from pygame.rect import Rect
from pygame.surface import Surface

from state import World
from state.constants import LAYER_GROUND_EARTH, LAYER_GROUND_SEA


class UserInterface:
    def __init__(self, world: World):
        self.__world = world

        # Create window with default resolution
        pygame.init()
        self.__window = pygame.display.set_mode((1024, 768), HWSURFACE | DOUBLEBUF | RESIZABLE)
        pygame.display.set_caption("2D Medieval Strategy Game with Python, http://www.patternsgameprog.com")
        pygame.display.set_icon(pygame.image.load("assets/toen/icon.png"))

        # Rendering properties
        self.__tileset = pygame.image.load("assets/toen/ground.png")
        self.__tileWidth = 16
        self.__tileHeight = 16
        self.__tiles = {
            LAYER_GROUND_EARTH: (2, 7),
            LAYER_GROUND_SEA: (5, 7),
        }
        self.__clock = pygame.time.Clock()

    def run(self):
        # Main game loop
        running = True
        while running:

            # Handle input
            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    running = False
                    break
                elif event.type == pygame.KEYDOWN:
                    if event.key == pygame.K_ESCAPE:
                        running = False
                        break

            # Render world in a surface
            renderWidth = self.__world.width * self.__tileWidth
            renderHeight = self.__world.height * self.__tileHeight
            renderSurface = Surface((renderWidth, renderHeight))
            tileWidth = self.__tileWidth
            tileHeight = self.__tileHeight
            for y in range(self.__world.height):
                for x in range(self.__world.width):
                    tile = self.__tiles[self.__world.getValue(x, y)]
                    tileRect = Rect(
                        tile[0] * tileWidth, tile[1] * tileHeight,
                        tileWidth, tileHeight
                    )
                    tileCoords = (
                        x * tileWidth,
                        y * tileHeight
                    )
                    renderSurface.blit(self.__tileset, tileCoords, tileRect)

            # Scale rendering to window size
            windowWidth, windowHeight = self.__window.get_size()
            renderRatio = renderWidth / renderHeight
            windowRatio = windowWidth / windowHeight
            if windowRatio <= renderRatio:
                rescaledSurfaceWidth = windowWidth
                rescaledSurfaceHeight = int(windowWidth / renderRatio)
                rescaledSurfaceX = 0
                rescaledSurfaceY = (windowHeight - rescaledSurfaceHeight) // 2
            else:
                rescaledSurfaceWidth = int(windowHeight * renderRatio)
                rescaledSurfaceHeight = windowHeight
                rescaledSurfaceX = (windowWidth - rescaledSurfaceWidth) // 2
                rescaledSurfaceY = 0

            # Scale the rendering to the window/screen size
            rescaledSurface = pygame.transform.scale(
                renderSurface, (rescaledSurfaceWidth, rescaledSurfaceHeight)
            )
            self.__window.blit(rescaledSurface, (rescaledSurfaceX, rescaledSurfaceY))
            pygame.display.update()
            self.__clock.tick(30)

    def quit(self):
        pygame.quit()
